package com.lovecoding.jdbc.service;

import com.lovecoding.jdbc.anno.Logs;
import com.lovecoding.jdbc.dao.Book;
import com.lovecoding.jdbc.dao.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
public class UserBuyBookServiceImpl implements UserBuyBookService{

    /**
     * 买书 分为一下及步骤
     * 0 查库存
     * 1 扣库存
     * 2 扣用户余额
     * @param uid
     * @param bookid
     */

    @Autowired
    private Book book;
    @Autowired
    private User user;
    @Autowired
    private TestService testService;
    /**
     * isolation
     * 事务隔离级别 是处理多事务， 多个事务交叉的时候隔离事务用的参数
     * 读未提交， 指的时候 一个事务可以读到 另一个事务未提交的修改内容
     * 读已提交， 同样指的是 一个事务可以读到 另一个事务已经提交的修改
     * 可重复读， 指的是 一个事务， 第一次读的东西， 被其他事务修改提交后， 依然读的是最初的值
     *  串行话， 所有事务排队执行
     */
    @Override
    @Transactional
    public void buyBook(int uid, int bookid) {
        //查询书的价格
        int price = book.getBook(bookid);
//        try {
//            Thread.sleep(5000);
//        } catch (InterruptedException e) {
//            e.printStackTrace();
//        }
        //扣减书的库存
        this.book.subBook(bookid);
        /**
         * 我们在公司 开发项目， 会有很多人协作
         * 每个人负责不同的代码， 最终代码混织在一起
         * 这个时候， 别人的代码就有可能会抛异常， 但是这个异常可能不是我们关注的业务
         * 为了不影响到自己的业务逻辑，所以我们可能会自己处理自己的异常
         * 指定抛出异常的 异常业务类
         */
        //扣减用户钱包
        user.balance(price, uid);

        //int n = 1 / 0;

        try {
            testService.updateTest();
        } catch (ArithmeticException e) {
            System.out.println( e.getMessage() );
        }

        System.out.println( "购买书本成功！" );

        /**
         * 事务传播， 也叫事务交叉
         * 这块涉及到 Spring事务管理， 是如何管理事务的、
         * 事务交叉， 我们开发项目， 经常和多个人协同处理
         * 如果别人也开了一个事务， 那么我们怎么处理事务呢
         * @T
         * buyBook(){
         *     @T
         *     bugBook()
         * }
         * 默认嵌套事务的时候，事务会互相影响
         * 一个事务里面嵌套 另一个事务， 实际上是一个事务
         */
    }
}
